""" Test rotations

Run the tests with::

    pytest nipraxis/tests/test_rotations.py
"""

import numpy as np

from nipraxis.rotations import x_rotmat, y_rotmat, z_rotmat

from numpy.testing import assert_almost_equal

EXAMPLE_ROTATIONS = np.linspace(0, np.pi * 5, 100)
RIGHT_ANGLE = np.pi / 2


def test_rotmats():
    # Test rotation functions
    for rot_func in (x_rotmat, y_rotmat, z_rotmat):
        # Check rotation of 0 degrees gives identity matrix
        assert_almost_equal(rot_func(0), np.eye(3))
        # On example rotations, check positive then negative rotation gives
        # identity matrix (0 degree rotation)
        for theta in EXAMPLE_ROTATIONS:
            M = rot_func(theta)
            iM = rot_func(-theta)
            assert_almost_equal(iM.dot(M), np.eye(3))
        # Test rotating twice by 90 degrees gives 180 degree rotation
        M = rot_func(RIGHT_ANGLE)
        assert_almost_equal(M.dot(M), rot_func(np.pi))
        # Test rotating four times by 90 degrees gives 0 degree rotation
        assert_almost_equal(M.dot(M).dot(M).dot(M), np.eye(3))



""" Nipraxis utilities

Nipraxis contains a series of utilities for the Nipraxis course at
https://nipraxis.org

The key function is ``fetch_file``; this fetches data from the nipraxis data
repository, caching in a known location to avoid repeat downloads.

Additional modules are:

* `stimuli``: functions to operate on FMRI run stimulus files.
* `rotations``: functions to work with roations in 3D.
"""

__version__ = '0.4'

from ._fetcher import fetch_file
from . import stimuli
from . import rotations



""" Rotation matrices for rotations around x, y, z axes

See: https://en.wikipedia.org/wiki/Rotation_matrix#Basic_rotations
"""

import numpy as np


def x_rotmat(theta):
    """ Rotation matrix for rotation of `theta` radians around x axis

    Parameters
    ----------
    theta : scalar
        Rotation angle in radians

    Returns
    -------
    M : shape (3, 3) array
        Rotation matrix
    """
    cos_t = np.cos(theta)
    sin_t = np.sin(theta)
    return np.array([[1, 0, 0],
                     [0, cos_t, -sin_t],
                     [0, sin_t, cos_t]])


def y_rotmat(theta):
    """ Rotation matrix for rotation of `theta` radians around y axis

    Parameters
    ----------
    theta : scalar
        Rotation angle in radians

    Returns
    -------
    M : shape (3, 3) array
        Rotation matrix
    """
    cos_t = np.cos(theta)
    sin_t = np.sin(theta)
    return np.array([[cos_t, 0, sin_t],
                     [0, 1, 0],
                     [-sin_t, 0, cos_t]])


def z_rotmat(theta):
    """ Rotation matrix for rotation of `theta` radians around z axis

    Parameters
    ----------
    theta : scalar
        Rotation angle in radians

    Returns
    -------
    M : shape (3, 3) array
        Rotation matrix
    """
    cos_t = np.cos(theta)
    sin_t = np.sin(theta)
    return np.array([[cos_t, -sin_t, 0],
                     [sin_t, cos_t, 0],
                     [0, 0, 1]])



""" Fetch data from repository, or maybe local cache
"""

import pkg_resources

import unscrewed

_config_file = pkg_resources.resource_filename("nipraxis", "registry.yaml")

fetcher = unscrewed.Fetcher(_config_file)
fetch_file = fetcher.fetch_file
